@@ -130,11 +130,7 @@ pub trait Encoder: Sized {
     /// Encoder that can encode a tuple struct.
     type TupleStruct: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder for a struct variant.
-    type StructVariant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
-    /// Encoder for a tuple variant.
-    type TupleVariant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
-    /// Encoder for a unit variant.
-    type UnitVariant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type Variant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
 
     /// An expectation error. Every other implementation defers to this to
     /// report that something unexpected happened.
@@ -166,163 +162,6 @@ pub trait Encoder: Sized {
         )))
     }
 
-    /// Construct a pack that can encode more than one element at a time.
-    ///
-    /// This hints to the format that it should attempt to encode all of the
-    /// elements in the packed sequence as compact as possible and that
-    /// subsequent unpackers will know the exact length of the element being
-    /// unpacked.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use musli::en::{Encode, Encoder, SequenceEncoder};
-    ///
-    /// struct PackedStruct {
-    ///     field: u32,
-    ///     data: [u8; 364],
-    /// }
-    ///
-    /// impl<Mode> Encode<Mode> for PackedStruct {
-    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
-    ///     where
-    ///         E: Encoder
-    ///     {
-    ///         let mut pack = encoder.encode_pack()?;
-    ///         pack.next()?.encode_u32(self.field)?;
-    ///         pack.next()?.encode_array(self.data)?;
-    ///         pack.end()
-    ///     }
-    /// }
-    /// ```
-    #[inline]
-    fn encode_pack(self) -> Result<Self::Pack, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::Pack,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
-    /// Encode fixed-length array.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use musli::en::{Encode, Encoder};
-    ///
-    /// struct MyType {
-    ///     data: [u8; 364],
-    /// }
-    ///
-    /// impl<Mode> Encode<Mode> for MyType {
-    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
-    ///     where
-    ///         E: Encoder
-    ///     {
-    ///         encoder.encode_array(self.data)
-    ///     }
-    /// }
-    /// ```
-    #[inline]
-    fn encode_array<const N: usize>(self, _: [u8; N]) -> Result<Self::Ok, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::Array,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
-    /// Encode a sequence of bytes.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use musli::en::{Encode, Encoder};
-    ///
-    /// struct MyType {
-    ///     data: Vec<u8>,
-    /// }
-    ///
-    /// impl<Mode> Encode<Mode> for MyType {
-    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
-    ///     where
-    ///         E: Encoder
-    ///     {
-    ///         encoder.encode_bytes(self.data.as_slice())
-    ///     }
-    /// }
-    /// ```
-    #[inline]
-    fn encode_bytes(self, _: &[u8]) -> Result<Self::Ok, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::Bytes,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
-    /// Encode the given slices of bytes in sequence, with one following another
-    /// as a single contiguous byte array.
-    ///
-    /// This can be useful to avoid allocations when a caller doesn't have
-    /// access to a single byte sequence like in
-    /// [VecDeque][std::collections::VecDeque].
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use std::collections::VecDeque;
-    ///
-    /// use musli::en::{Encode, Encoder};
-    ///
-    /// struct MyType {
-    ///     data: VecDeque<u8>,
-    /// }
-    ///
-    /// impl<Mode> Encode<Mode> for MyType {
-    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
-    ///     where
-    ///         E: Encoder
-    ///     {
-    ///         let (first, second) = self.data.as_slices();
-    ///         encoder.encode_bytes_vectored(&[first, second])
-    ///     }
-    /// }
-    /// ```
-    #[inline]
-    fn encode_bytes_vectored(self, _: &[&[u8]]) -> Result<Self::Ok, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::Bytes,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
-    /// Encode a string.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use musli::en::{Encode, Encoder};
-    ///
-    /// struct MyType {
-    ///     data: String,
-    /// }
-    ///
-    /// impl<Mode> Encode<Mode> for MyType {
-    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
-    ///     where
-    ///         E: Encoder
-    ///     {
-    ///         encoder.encode_string(self.data.as_str())
-    ///     }
-    /// }
-    /// ```
-    #[inline]
-    fn encode_string(self, _: &str) -> Result<Self::Ok, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::String,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
     /// Encode a boolean value.
     ///
     /// # Examples
@@ -771,6 +610,126 @@ pub trait Encoder: Sized {
         )))
     }
 
+    /// Encode fixed-length array.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use musli::en::{Encode, Encoder};
+    ///
+    /// struct MyType {
+    ///     data: [u8; 364],
+    /// }
+    ///
+    /// impl<Mode> Encode<Mode> for MyType {
+    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
+    ///     where
+    ///         E: Encoder
+    ///     {
+    ///         encoder.encode_array(self.data)
+    ///     }
+    /// }
+    /// ```
+    #[inline]
+    fn encode_array<const N: usize>(self, _: [u8; N]) -> Result<Self::Ok, Self::Error> {
+        Err(Self::Error::message(InvalidType::new(
+            expecting::Array,
+            &ExpectingWrapper::new(self),
+        )))
+    }
+
+    /// Encode a sequence of bytes.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use musli::en::{Encode, Encoder};
+    ///
+    /// struct MyType {
+    ///     data: Vec<u8>,
+    /// }
+    ///
+    /// impl<Mode> Encode<Mode> for MyType {
+    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
+    ///     where
+    ///         E: Encoder
+    ///     {
+    ///         encoder.encode_bytes(self.data.as_slice())
+    ///     }
+    /// }
+    /// ```
+    #[inline]
+    fn encode_bytes(self, _: &[u8]) -> Result<Self::Ok, Self::Error> {
+        Err(Self::Error::message(InvalidType::new(
+            expecting::Bytes,
+            &ExpectingWrapper::new(self),
+        )))
+    }
+
+    /// Encode the given slices of bytes in sequence, with one following another
+    /// as a single contiguous byte array.
+    ///
+    /// This can be useful to avoid allocations when a caller doesn't have
+    /// access to a single byte sequence like in
+    /// [VecDeque][std::collections::VecDeque].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::collections::VecDeque;
+    ///
+    /// use musli::en::{Encode, Encoder};
+    ///
+    /// struct MyType {
+    ///     data: VecDeque<u8>,
+    /// }
+    ///
+    /// impl<Mode> Encode<Mode> for MyType {
+    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
+    ///     where
+    ///         E: Encoder
+    ///     {
+    ///         let (first, second) = self.data.as_slices();
+    ///         encoder.encode_bytes_vectored(&[first, second])
+    ///     }
+    /// }
+    /// ```
+    #[inline]
+    fn encode_bytes_vectored(self, _: &[&[u8]]) -> Result<Self::Ok, Self::Error> {
+        Err(Self::Error::message(InvalidType::new(
+            expecting::Bytes,
+            &ExpectingWrapper::new(self),
+        )))
+    }
+
+    /// Encode a string.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use musli::en::{Encode, Encoder};
+    ///
+    /// struct MyType {
+    ///     data: String,
+    /// }
+    ///
+    /// impl<Mode> Encode<Mode> for MyType {
+    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
+    ///     where
+    ///         E: Encoder
+    ///     {
+    ///         encoder.encode_string(self.data.as_str())
+    ///     }
+    /// }
+    /// ```
+    #[inline]
+    fn encode_string(self, _: &str) -> Result<Self::Ok, Self::Error> {
+        Err(Self::Error::message(InvalidType::new(
+            expecting::String,
+            &ExpectingWrapper::new(self),
+        )))
+    }
+
     /// Encode an optional value that is present.
     ///
     /// # Examples
@@ -841,6 +800,43 @@ pub trait Encoder: Sized {
         )))
     }
 
+    /// Construct a pack that can encode more than one element at a time.
+    ///
+    /// This hints to the format that it should attempt to encode all of the
+    /// elements in the packed sequence as compact as possible and that
+    /// subsequent unpackers will know the exact length of the element being
+    /// unpacked.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use musli::en::{Encode, Encoder, SequenceEncoder};
+    ///
+    /// struct PackedStruct {
+    ///     field: u32,
+    ///     data: [u8; 364],
+    /// }
+    ///
+    /// impl<Mode> Encode<Mode> for PackedStruct {
+    ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
+    ///     where
+    ///         E: Encoder
+    ///     {
+    ///         let mut pack = encoder.encode_pack()?;
+    ///         pack.next()?.encode_u32(self.field)?;
+    ///         pack.next()?.encode_array(self.data)?;
+    ///         pack.end()
+    ///     }
+    /// }
+    /// ```
+    #[inline]
+    fn encode_pack(self) -> Result<Self::Pack, Self::Error> {
+        Err(Self::Error::message(InvalidType::new(
+            expecting::Pack,
+            &ExpectingWrapper::new(self),
+        )))
+    }
+
     /// Encode a sequence with a known length.
     ///
     /// # Examples
@@ -1009,7 +1005,7 @@ pub trait Encoder: Sized {
     /// enum Enum {
     ///     UnitVariant,
     ///     TupleVariant(String),
-    ///     StructVariant {
+    ///     Variant {
     ///         data: String,
     ///         age: u32,
     ///     }
@@ -1020,17 +1016,16 @@ pub trait Encoder: Sized {
     ///     where
     ///         E: Encoder
     ///     {
+    ///         let mut variant = encoder.encode_variant()?;
+    ///
     ///         match self {
     ///             Enum::UnitVariant => {
-    ///                 let variant = encoder.encode_unit_variant()?;
     ///                 variant.insert::<Mode, _, _>("variant1", ())
     ///             }
     ///             Enum::TupleVariant(data) => {
-    ///                 let variant = encoder.encode_tuple_variant(1)?;
     ///                 variant.insert::<Mode, _, _>("variant2", data)
     ///             }
-    ///             Enum::StructVariant { data, age } => {
-    ///                 let mut variant = encoder.encode_struct_variant(2)?;
+    ///             Enum::Variant { data, age } => {
     ///                 variant.first()?.encode_string("variant3")?;
     ///
     ///                 let mut st = variant.second()?.encode_struct(2)?;
@@ -1045,27 +1040,9 @@ pub trait Encoder: Sized {
     /// }
     /// ```
     #[inline]
-    fn encode_struct_variant(self, _: usize) -> Result<Self::StructVariant, Self::Error> {
+    fn encode_variant(self) -> Result<Self::Variant, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
-            expecting::StructVariant,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
-    /// Encode an tuple enum variant.
-    #[inline]
-    fn encode_tuple_variant(self, _: usize) -> Result<Self::TupleVariant, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::TupleVariant,
-            &ExpectingWrapper::new(self),
-        )))
-    }
-
-    /// Encode an unit enum variant.
-    #[inline]
-    fn encode_unit_variant(self) -> Result<Self::UnitVariant, Self::Error> {
-        Err(Self::Error::message(InvalidType::new(
-            expecting::UnitVariant,
+            expecting::Variant,
             &ExpectingWrapper::new(self),
         )))
     }
